# -*- coding: utf-8 -*-
"""
This module contains classes from the main app used by Pyto.

This module is only for internal or plugin use.
"""

if __platform__ is macOS:
    raise ImportError("`pyto` module can only be imported on iOS. To check for platform, see the `__platform__` global variable.")

from rubicon.objc import *

NSBundle = ObjCClass("NSBundle")

def __isMainApp__():
    return (NSBundle.mainBundle.bundleURL.lastPathComponent == "Pyto.app" or NSBundle.mainBundle.bundleURL.lastPathComponent == "Pyto Mac.app")

def __is_appex__():
    return (NSBundle.mainBundle.bundleURL.pathExtension == "appex")

def __Class__(name):
    try:
        if (__isMainApp__()):
            if __platform__ is iOS:
                return ObjCClass("Pyto."+name)
            elif __platform__ is macOS:
                return ObjCClass("Pyto_Mac."+name)
            else:
                return ObjCClass(name)
        elif __is_appex__():
            return ObjCClass("Pyto_Widget."+name)
        else:
            return ObjCClass("PytoCore."+name)
    except:
        return None

PyMainThread = __Class__("PyMainThread")
PyThread = __Class__("PyThread")

PyInputHelper = __Class__("PyInputHelper")
PyOutputHelper = __Class__("PyOutputHelper")
PySharingHelper = __Class__("PySharingHelper")
PySelector = __Class__("PySelector")
SelectorTarget = __Class__("_Selector")
FilePicker = __Class__("PyFilePicker")
PyAlert = __Class__("PyAlert")
Python = __Class__("Python")
ConsoleViewController = __Class__("ConsoleViewController")
QuickLookHelper = __Class__("QuickLookHelper")

if __isMainApp__():
    REPLViewController = __Class__("REPLViewController")
else:
    REPLViewController = None

if (__isMainApp__()):
    ReviewHelper = __Class__("ReviewHelper")
    EditorViewController = __Class__("EditorViewController")
    EditorSplitViewController = __Class__("EditorSplitViewController")
    DocumentBrowserViewController = __Class__("DocumentBrowserViewController")
else:
    ReviewHelper = None
    EditorViewController = None
    EditorSplitViewController = None
    DocumentBrowserViewController = None

all = ["PyMainThread", "PyThread", "PyInputHelper", "PyOutputHelper", "PySharingHelper", "PySelector", "SelectorTarget", "FilePicker", "PyAlert", "Python", "ConsoleViewController", "ReviewHelper", "EditorViewController", "EditorSplitViewController", "REPLViewController", "QuickLookHelper"]
